package java.time.zone;

import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;
import java.security.AccessController;
import java.util.List;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.concurrent.ConcurrentHashMap;
import java.util.Iterator;
import java.util.NavigableMap;
import java.util.Objects;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.CopyOnWriteArrayList;

public abstract class ZoneRulesProvider
{
    private static final CopyOnWriteArrayList<ZoneRulesProvider> PROVIDERS;
    private static final ConcurrentMap<String, ZoneRulesProvider> ZONES;
    
    public static Set<String> getAvailableZoneIds() {
        return new HashSet(ZoneRulesProvider.ZONES.keySet());
    }
    
    public static ZoneRules getRules(String obj, boolean b) {
        Objects.requireNonNull(obj, "zoneId");
        return getProvider(obj).provideRules(obj, b);
    }
    
    public static NavigableMap<String, ZoneRules> getVersions(String obj) {
        Objects.requireNonNull(obj, "zoneId");
        return getProvider(obj).provideVersions(obj);
    }
    
    private static ZoneRulesProvider getProvider(String str) {
        ZoneRulesProvider zoneRulesProvider = (ZoneRulesProvider)ZoneRulesProvider.ZONES.get(str);
        if (zoneRulesProvider == null) {
            if (ZoneRulesProvider.ZONES.isEmpty()) {
                throw new ZoneRulesException("No time-zone data files registered");
            }
            throw new ZoneRulesException("Unknown time-zone ID: " + str);
        }
        else {
            return zoneRulesProvider;
        }
    }
    
    public static void registerProvider(ZoneRulesProvider zoneRulesProvider) {
        Objects.requireNonNull(zoneRulesProvider, "provider");
        registerProvider0(zoneRulesProvider);
        ZoneRulesProvider.PROVIDERS.add(zoneRulesProvider);
    }
    
    private static void registerProvider0(ZoneRulesProvider obj) {
        for (String s : obj.provideZoneIds()) {
            Objects.requireNonNull(s, "zoneId");
            if ((ZoneRulesProvider)ZoneRulesProvider.ZONES.putIfAbsent(s, obj) != null) {
                throw new ZoneRulesException("Unable to register zone as one already registered with that ID: " + s + ", currently loading from provider: " + obj);
            }
            continue;
        }
    }
    
    public static boolean refresh() {
        boolean n = false;
        Iterator<ZoneRulesProvider> iterator = ZoneRulesProvider.PROVIDERS.iterator();
        while (iterator.hasNext()) {
            n |= ((ZoneRulesProvider)iterator.next()).provideRefresh();
        }
        return n;
    }
    
    protected ZoneRulesProvider() {
    }
    
    protected abstract Set<String> provideZoneIds();
    
    protected abstract ZoneRules provideRules(String p0, boolean p1);
    
    protected abstract NavigableMap<String, ZoneRules> provideVersions(String p0);
    
    protected boolean provideRefresh() {
        return false;
    }
    
    static {
        PROVIDERS = new CopyOnWriteArrayList();
        ZONES = new ConcurrentHashMap(512, 0.75f, 2);
        final ArrayList<ZoneRulesProvider> c = new ArrayList();
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                String property = System.getProperty("java.time.zone.DefaultZoneRulesProvider");
                if (property != null) {
                    try {
                        ZoneRulesProvider zoneRulesProvider = (ZoneRulesProvider)ZoneRulesProvider.class.cast(Class.forName(property, true, ClassLoader.getSystemClassLoader()).newInstance());
                        ZoneRulesProvider.registerProvider(zoneRulesProvider);
                        c.add(zoneRulesProvider);
                    }
                    catch (Exception cause) {
                        throw new Error(cause);
                    }
                }
                else {
                    ZoneRulesProvider.registerProvider(new TzdbZoneRulesProvider());
                }
                return null;
            }
        });
        Iterator<ZoneRulesProvider> iterator = ServiceLoader.load(ZoneRulesProvider.class, ClassLoader.getSystemClassLoader()).iterator();
        while (iterator.hasNext()) {
            ZoneRulesProvider zoneRulesProvider;
            try {
                zoneRulesProvider = (ZoneRulesProvider)iterator.next();
            }
            catch (ServiceConfigurationError serviceConfigurationError) {
                if ((serviceConfigurationError.getCause() instanceof SecurityException)) {
                    continue;
                }
                throw serviceConfigurationError;
            }
            boolean n = false;
            Iterator<Object> iterator2 = c.iterator();
            while (iterator2.hasNext()) {
                if (((ZoneRulesProvider)iterator2.next()).getClass() == zoneRulesProvider.getClass()) {
                    n = true;
                }
                else {
                    continue;
                }
            }
            if (n == 0) {
                registerProvider0(zoneRulesProvider);
                c.add(zoneRulesProvider);
            }
            else {
                continue;
            }
        }
        ZoneRulesProvider.PROVIDERS.addAll(c);
    }
}
